/* 	DriverView.java
 *	Copyright (c) 2004-2007, Brains2B.org
 *	
 *	Created by: Dennis Groenendijk
 *	Created on: Feb 11, 2004
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that  * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/* @#History
 * [When]		[Who]					[What]
 * 09-02-2007   dennis@brains2b.nl      Use for both frame and dialog, use dynamic classpath
 * 14-03-2007   dennis@brains2b.nl      Fixed problem with setting drivers after scan
 * 07-04-2007   dennis@brains2b.nl      Fixed problem with classPath file 		
*/
package org.brains2b.swingext.connect;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.Window;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Iterator;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.SwingUtilities;

import org.brains2b.jdbc.Driver;
import org.brains2b.loader.DynamicClassLoader;
import org.brains2b.swingext.form.Form;
import org.brains2b.swingext.form.FormAction;
import org.brains2b.swingext.gui.IconFactory;

/** [Short description]
 *  [Long description]
 *
 * @author <A HREF="MAILTO:dennis@brains2b.nl">dennis@brains2b.nl</A>
 * @version 0.10.3 [07-04-2007]
 */
public class DriverView extends JPanel implements Form {

	private class ActionHandler implements ActionListener {
		
		public void actionPerformed(ActionEvent arg0) {
			if ("classpath.Open".equals(arg0.getActionCommand())) {
				File f=null;
				if (m_driver!=null && m_driver.getClassPath()!=null) {
					f=new File(m_driver.getClassPath()[0].getFile());
				}
				JFileChooser jfc=new JFileChooser(f);
				jfc.setMultiSelectionEnabled(true);
				jfc.showOpenDialog(DriverView.this);
				File file[] = jfc.getSelectedFiles();
				for (int i=0;i<file.length;i++) {
					((DefaultListModel) classPath.getModel()).addElement(file[i]);
				}
				
			} else if ("classpath.Remove".equals(arg0.getActionCommand())) {
				int[]idx=classPath.getSelectedIndices();
				for (int i=0;i<idx.length;i++) {
					((DefaultListModel) classPath.getModel()).removeElementAt(idx[i]);
				}
			} else if ("classpath.Scan".equals(arg0.getActionCommand())) {
				int size = ((DefaultListModel) classPath.getModel()).size();
				URL[] path=null;
				if (size>0) {
					path=new URL[size];
					for (int i=0;i<size;i++) {
						try {
							path[i] = ((File) ((DefaultListModel) classPath.getModel()).getElementAt(i)).toURL();
						} catch (MalformedURLException muex) {
							message.setText("Not a valid classpath");
							return;
						}
					} 
					try {
						Class[] c=DynamicClassLoader.findImplementation(java.sql.Driver.class,path);
						if (c.length==0) {
							message.setText("classpath contains no drivers");
							return;
						}
						for (int i=0;i<c.length;i++) {
							driverName.addItem(c[i].getName());
						}
						driverName.setSelectedIndex(0);
					} catch (ClassNotFoundException cnfex) {
						message.setText("Error trying to find driver");
					}
					
				} else {
					message.setText("Please provide a classpath first.");
				}
			}
		}
	}
	private final static String TITLE="Driver information";

	private JComboBox driverName;
	private JTextField urlPrefix;
	private JLabel message;
	private JList classPath;
	
	private Vector m_drivers;
	private Driver m_driver;
	
	public DriverView() {
		super();
		init();
	}
	
	public static Driver showDailog(Window parent, Vector drivers, Driver d) {
		JDialog dlg=null;
        if (parent instanceof JDialog) {
            dlg = new JDialog((JDialog) parent,true);    
        } else {
            dlg = new JDialog((JFrame) parent, true); 
        }
        dlg.setTitle(TITLE);
		dlg.setLocationRelativeTo(parent);
		DriverView view=new DriverView();
		view.setDriver(d);
		view.setDrivers(drivers);
		dlg.setContentPane(view);
		dlg.pack();
		dlg.setVisible(true);
		return view.getDriver();
		
	}
	
	private void init() {
		setLayout(new BorderLayout());
		setBorder(BorderFactory.createEmptyBorder(5,5,5,5));
		JPanel main=new JPanel(new GridBagLayout());
		GridBagConstraints gbc=new GridBagConstraints();
		ActionHandler ah=new ActionHandler();
		gbc.insets=new Insets(5,5,5,5);
		gbc.anchor=GridBagConstraints.WEST;
		gbc.gridy=0;
		gbc.gridx=0;
		
		main.add(new JLabel("driver className"),gbc);
		driverName=new JComboBox();
		driverName.setEditable(true);
		gbc.fill=GridBagConstraints.HORIZONTAL;
		gbc.gridx++;
		main.add(driverName,gbc);
		
		gbc.gridx=0;
		gbc.gridy++;	
		main.add(new JLabel("url prefix"),gbc);
		gbc.gridx++;
		urlPrefix=new JTextField();
		urlPrefix.setColumns(20);
		main.add(urlPrefix,gbc);
		
		gbc.gridx=0;
		gbc.gridy++;
		main.add(new JLabel("classpath"),gbc);
		gbc.gridx++;
		gbc.gridheight=5;
		gbc.fill=GridBagConstraints.BOTH;
		classPath=new JList();
        classPath.setFixedCellWidth(256);
		classPath.setModel(new DefaultListModel());
		main.add(new JScrollPane(classPath),gbc);
		gbc.gridheight=1;
		gbc.gridx++;
		gbc.insets=new Insets(0,5,0,5);
		JButton btnAdd=new JButton(IconFactory.createIcon("/org/brains2b/images/Open16.gif"));
		btnAdd.setActionCommand("classpath.Open");
		btnAdd.addActionListener(ah);
		main.add(btnAdd,gbc);
		gbc.gridy++;
		JButton btnRem=new JButton(IconFactory.createIcon("/org/brains2b/images/Minus16.gif"));
		btnRem.addActionListener(ah);
		btnRem.setActionCommand("classpath.Remove");
		main.add(btnRem,gbc);
		gbc.gridy++;
		JButton btnScan=new JButton(IconFactory.createIcon("/org/brains2b/images/Find16.png"));
		btnScan.addActionListener(ah);
		btnScan.setActionCommand("classpath.Scan");
		main.add(btnScan,gbc);
		gbc.insets=new Insets(5,5,5,5);
		
		gbc.gridy+=5;
		gbc.gridx=0;
		gbc.gridwidth=2;
		gbc.gridheight=1;
		message=new JLabel(" ");
		message.setForeground(Color.red);
		main.add(message,gbc);
		
		add(main,BorderLayout.CENTER);
		FormAction fa= new FormAction(this, FormAction.BTN_OK_CANCEL+FormAction.BTN_UNDO);
		fa.getButton(FormAction.BTN_UNDO).setText("Delete");
		add(fa, BorderLayout.SOUTH);
		
	}
	/* (non-Javadoc)
	 * @see org.brains2b.model.gui.Form#performApply()
	 */
	public void performApply() {
//		Not implemented
	}

	/* (non-Javadoc)
	 * @see org.brains2b.model.gui.Form#performOk()
	 */
	public void performOk() {
		String name=driverName.getSelectedItem().toString();
		if (name.trim().length()==0) name=null;
		
		if (name!=null) {
			int size = ((DefaultListModel) classPath.getModel()).size();
			URL[] path=null;
			if (size>0) {
				path=new URL[size];
				for (int i=0;i<size;i++) {
					try {
						path[i] = ((File) ((DefaultListModel) classPath.getModel()).getElementAt(i)).toURL();
					} catch (MalformedURLException muex) {
						message.setText("Not a valid classpath");
					}
				}
			}
			try {
				
				DynamicClassLoader.loadClass(name,path);
			} catch (ClassNotFoundException cnfex) {
				message.setText("Driver not found in classpath, cannot add driver");
				return;
			}
			if (m_driver==null && m_drivers!=null) {
				Iterator it=m_drivers.iterator();
				while (it.hasNext()) {
					Driver d=(Driver) it.next();
					if (name.equals(d.getClassName())) {
						//TODO What if you need two separate implementations of driver
						message.setText("Driver is already defined");
						return;
					}
				}
			}
			if (m_driver==null) {
				m_driver=new Driver(name,urlPrefix.getText());
			} else {
				m_driver.setClassName(name);
				m_driver.setUrlprefix(urlPrefix.getText());
			}
			m_driver.setClassPath(path);
			close();
		} else {
			message.setText("Enter a driver name, or press cancel");
			return;
		}
	}

	/* (non-Javadoc)
	 * delete the current driver
	 * @see org.brains2b.model.gui.Form#performUndo()
	 */
	public void performUndo() {
		m_driver.setClassName("DELETED");
		close();
	}

	/* (non-Javadoc)
	 * @see org.brains2b.model.gui.Form#performCancel()
	 */
	public void performCancel() {
		m_driver=null;
		close();

	}

	/**
	 * @return
	 */
	public Driver getDriver() {
		return m_driver;
	}

	/**
	 * @param driver
	 */
	public void setDriver(Driver driver) {
		this.m_driver = driver;
		if (driver!=null) {
			driverName.setSelectedItem(driver.getClassName());
			urlPrefix.setText(driver.getUrlprefix());
			if (driver.getClassPath()!=null) {
				for (int i=0;i<driver.getClassPath().length;i++)
				((DefaultListModel) classPath.getModel()).addElement(new File(driver.getClassPath()[i].getPath()));
			}
		}
	}
	
	/**
	 * @param m_driver
	 */
	public void setDrivers(Vector drivers) {
		this.m_drivers = drivers;
	}

	private void close() {
		Window w= SwingUtilities.windowForComponent(this);
		if (w instanceof JDialog) {
			((JDialog) w).setVisible(false);
			((JDialog) w).dispose();
			
		}
	}
}
