/* 	DialogHelper.java
 *	Copyright (c) 2007, Brains2B.org
 *	
 *	Created by: Dennis Groenendijk
 *	Created on: Jul 6, 2007
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that  * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package org.brains2b.swingext.dialog;

import java.awt.Component;
import java.awt.Container;
import java.awt.Dialog;
import java.awt.Frame;
import java.awt.Window;

import javax.swing.Icon;
import javax.swing.JDesktopPane;
import javax.swing.JDialog;
import javax.swing.JInternalFrame;
import javax.swing.JLayeredPane;
import javax.swing.SwingUtilities;

import org.brains2b.util.GuiHelper;

/** [Short description]
 *  [Long description]
 *
 * @author <A HREF="MAILTO:dennis@brains2b.nl">dennis@brains2b.nl</A>
 * @version [version] [Jul 6, 2007]
 */
public class DialogHelper {

    /**
     * shows a JDialog on the Window in the hierarchy of the component supplied
     * @param component Component, the component on which this object should be supplied
     * @return JDialog
     */
    public final static void showDialog(Component parent, Container pane, String title) {
        Window window = DialogHelper.getWindowPane(parent);
        JDialog dialog = null;
        if (window instanceof Frame || window == null) {
            dialog = new JDialog((Frame) window, true);
        } else {
            dialog = new JDialog((Dialog) window, true);
        }
        
        dialog.setContentPane(pane);
        dialog.setTitle(title);
        
        dialog.pack();
        dialog.setBounds(GuiHelper.setMiddle(dialog.getBounds(),window.getBounds()));
        dialog.setResizable(false);
        dialog.show();
    }
    
    /**
     * shows a JInternalFrame on the Window in the hierarchy of the component supplied
     * @param component Component, the component on which this object should be supplied
     * @return JDialog
     */
    public final static void showInternalDialog(Component parent, Container pane, String title, Icon icon) {
        JDesktopPane desktop = getDesktopPane(parent);
        if (desktop == null) {
            throw new RuntimeException("parent has no valid JDesktopPane");
        }
        final JInternalFrame frame = new JInternalFrame();
        desktop.add(frame, JLayeredPane.MODAL_LAYER);
        frame.setContentPane(pane);
        frame.setTitle(title);
        if (icon!=null) {
            frame.setFrameIcon(icon);
        }
        frame.pack();
        frame.setBounds(GuiHelper.setMiddle(frame.getBounds(),desktop.getBounds()));
        try {
            frame.setVisible(true);
            frame.setSelected(true);
        } catch (java.beans.PropertyVetoException e) {
        }
        
        //The internal frame is not really modal, from 1.5 we will use the setLWModal()
        //but now we will fake it
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                while (!frame.isVisible()) {
                    try {
                        wait(100);
                    } catch(Exception ex) {}
                    notify();
                }
            }
        });
        
    }
    /**
     * get the Desktoppane which is ancestor of a component
     * @param parent Component, the component to find the JDesktopPane for
     * @return JDesktopPane, the pane found in the hierarchy of the component,
     * null in none was found.
     */
    public final static JDesktopPane getDesktopPane(Component parent) {
        if (parent instanceof Window) {
            throw new IllegalArgumentException("You cannot supply an ancestor of the JDesktop pane as parent");
        }
    	if (parent == null)
    		return null;
    	if (parent instanceof JDesktopPane) {
    		return (JDesktopPane) parent;
        } else {
    		return getDesktopPane(parent.getParent());
    	}
    }

    /**
     * get the Window which is ancestor of a component
     * @param parent Component, the component to find the Window for
     * @return Window, the window found in the hierarchy of the component,
     * null in none was found.
     */
    public final static Window getWindowPane(Component parent) {
    	if (parent == null) {
    		return null;
    	} else if (parent instanceof Frame || parent instanceof Dialog) {
    		return (Window) parent;
    	} else {
    		return getWindowPane(parent.getParent());
    	}
    }

    
}
