/* 	EditTextPanel.java
 * 
 * 	Created on: Jun 1, 2003
 * 	Created by: Dennis Groenendijk
 * 
 * Copyright (c) 2003, Brains2B.org
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that
 * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
 /* @#History
 * [When]		[Who]				[What]
 * 01-Oct-03	dennis@brains2b.nl	Fixed returning carriage returns to a JTextField
 *
 */
package org.brains2b.swingext.dialog;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.util.Locale;
import java.util.ResourceBundle;

import javax.swing.JInternalFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.JTextPane;
import javax.swing.SwingUtilities;
import javax.swing.border.BevelBorder;
import javax.swing.border.CompoundBorder;
import javax.swing.border.EmptyBorder;
import javax.swing.text.JTextComponent;

import org.brains2b.swingext.form.Form;
import org.brains2b.swingext.form.FormAction;

/** Panel for showing a larger TextPanel to edit text in.
 * 
 * @author <A HREF="MAILTO:dennis@brains2b.nl">dennis@brains2b.nl</A>
 * @version 0.1 [1-6-2003]
 */
public class EditorPane extends JPanel implements Form {
    
    
	private JTextComponent m_txtComp;
	private JTextPane m_textArea;
	private int maxLength=-1;
	/**
	 * Constructor for EditDialog.
	 * @param owner
	 */
	public EditorPane(JTextComponent txtComponent) {
		super();
		m_txtComp=txtComponent;
		init();
	}

	private void init() {
		setLayout(new BorderLayout());
		FormAction fa=new FormAction(
		        this,
		        FormAction.BTN_OK_CANCEL,
		        ResourceBundle.getBundle("org.brains2b.swingext.resource.dialog"));
		add(fa,BorderLayout.SOUTH);
		m_textArea=new JTextPane();
		JScrollPane scroll=new JScrollPane();
		scroll.setBorder(new CompoundBorder(new EmptyBorder(5,5,5,5),new BevelBorder(BevelBorder.LOWERED)));
		scroll.setViewportView(m_textArea);
		scroll.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
		scroll.setPreferredSize(getOneFifth());
		add(scroll,BorderLayout.CENTER);
		if (getRootPane() != null) {
			getRootPane().setDefaultButton(fa.getButton(FormAction.BTN_OK));
		}
		m_textArea.setText(m_txtComp.getText());
	}
	
	/**
	 * @see org.brains2b.model.gui.Form#performApply()
	 */
	public void performApply() {
		//not implemented
	}

	/**
	 * On perform Ok the text is set to the JTextComponent 
	 * which was registered to the EditorPane. If the
	 * JTextComponent is a textfield carriage returns are replaced
	 * by spaces while JTextField cannot contain carriage returns
	 * @see org.brains2b.model.gui.Form#performOk()
	 */
	public void performOk() {
		if (m_txtComp!=null) {
			if (maxLength>0) {
				int length=m_textArea.getText().length();
				if (length>maxLength) {
					ResourceBundle b=ResourceBundle.getBundle("org.brains2b.resource.messages",Locale.getDefault());
					JOptionPane.showMessageDialog(this,b.getString("EditorPane.length"));
					m_textArea.select(maxLength, length);
					m_textArea.requestFocus();
					return;
				}
			}
			String text=m_textArea.getText();
			if (m_txtComp instanceof JTextField){
				// remove returns from text
				text=text.replace((char) 0x0A, (char) 0x20);
			}
			m_txtComp.setText(text);
		}
		
		close();
	
	}

	/**
	 * @see org.brains2b.model.gui.Form#performUndo()
	 */
	public void performUndo() {
		//not implemented
	}

	/**
	 * @see org.brains2b.model.gui.Form#performCancel()
	 */
	public void performCancel() {
		close();
	}

	private Dimension getOneFifth() {
		Dimension d=this.getToolkit().getScreenSize();
		d.height=d.height/5;
		d.width=d.width/3;
		return d;
	}
	
	public static JTextComponent showDialog(Component parent,JTextComponent cmp, String title) {
			Object large=cmp.getClientProperty("large");
			
			EditorPane edit=new EditorPane(cmp);
			if (large!=null) {
				if (large instanceof String && Character.isDigit(((String)large).charAt(0))) {
					edit.setMaxLength(Integer.parseInt((String)large));
				} else if (large instanceof Integer) {
					edit.setMaxLength(((Integer)large).intValue());
				}
			}
			DialogHelper.showDialog(parent, edit, title);
			return edit.m_txtComp;
	} 
	
	public static JTextComponent showInternalDialog(Component parent,JTextComponent cmp, String title) {
			
			Object large=cmp.getClientProperty("large");
			
			EditorPane edit=new EditorPane(cmp);
			if (large!=null) {
				if (large instanceof String && Character.isDigit(((String)large).charAt(0))) {
					edit.setMaxLength(Integer.parseInt((String)large));
				} else if (large instanceof Integer) {
					edit.setMaxLength(((Integer)large).intValue());
				}
			}
			DialogHelper.showInternalDialog(parent, edit, title, null);
            
            return edit.m_txtComp;
	} 
		
	/**
     * returns maximum length of characters that can be
     * edited by this EditorPane
	 * @return int 
	 */
	public int getMaxLength() {
		return maxLength;
	}

	/**
	 * @param i
	 */
	public void setMaxLength(int i) {
		maxLength = i;
	}


    private void close() {
        if (getRootPane().getParent() instanceof JInternalFrame) {
            ((JInternalFrame) getRootPane().getParent()).doDefaultCloseAction();      
        } else {
            SwingUtilities.getWindowAncestor(this).dispose();
        }
    }

}
