/*  Find.java
 *  Copyright (c) 1999-2001, Dennis Groenendijk
 *  Copyright (c) 2002-2007, Brains2B.org
 *  
 *  Created by: Dennis Groenendijk
 *  Created on: May 3, 1999
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that  * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package org.brains2b.swingext.find;

import java.awt.BorderLayout;
import java.awt.FlowLayout;

import javax.swing.BoxLayout;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

import org.brains2b.swingext.form.Form;
import org.brains2b.swingext.form.FormAction;

public class FindPane extends JPanel {
	
	public static final int FIND=1;
	public static final int REPLACE=2;
	
    private int m_type;
	private Form m_handler;
    private String m_title;
    
	private JTextField txtFind;
	private JTextField txtReplace;
	
	public FindPane(Form handler) {
		this(handler,FIND);
	}
	
	public FindPane(Form handler, int type) {
		super();
        m_handler = handler;
        m_type = type;
		init();
	}
	
	private void init() {
        setLayout(new BorderLayout());
		setTitle(m_type);
		
        FormAction fa =new FormAction(m_handler,
                m_type==FIND?FormAction.BTN_OK_CANCEL+FormAction.BTN_APPLY:FormAction.BTN_ALL);
		if (m_type==REPLACE) {
 			add(getReplace(),BorderLayout.CENTER);
            fa.getButton(FormAction.BTN_APPLY).setText("Replace");
            fa.getButton(FormAction.BTN_UNDO).setText("Replace All");
		} else {
			add(getFind(),BorderLayout.CENTER);
            fa.getButton(FormAction.BTN_APPLY).setText("Find");
 		}
        add(fa, BorderLayout.SOUTH);
		
	}
	
	private JPanel getFind() {
		JPanel pnl=new JPanel(new FlowLayout(FlowLayout.RIGHT));
		txtFind=new JTextField();
		txtFind.setColumns(20);
		
		JLabel lbl=new JLabel("Find");
		lbl.setDisplayedMnemonic('f');
		lbl.setLabelFor(txtFind);

		pnl.add(lbl);
		pnl.add(txtFind);

		return pnl;
	}
	
	private JPanel getReplace() {
		JPanel root=new JPanel();
		root.setLayout(new BoxLayout(root,BoxLayout.Y_AXIS));
		root.add(getFind());
		JPanel pnl=new JPanel(new FlowLayout(FlowLayout.RIGHT));
		txtReplace=new JTextField();
		txtReplace.setColumns(20);
	
		JLabel lbl=new JLabel("Replace with");
		lbl.setDisplayedMnemonic('r');
		lbl.setLabelFor(txtReplace);

		pnl.add(lbl);
		pnl.add(txtReplace);
		root.add(pnl);
		return root;
	}
	
	private void setTitle(int type) {
		if(type==FIND) {
			setTitle("Find ...");
		} else {
			setTitle("Replace ...");
		}
	}
    
    public void setTitle(String title) {
        m_title = title;
    }
    
    public String getTitle() {
        return m_title;
    }
	
	public Find getResult() {
		if (txtFind.getText().length()>0) {
			return new Find(txtFind.getText(), txtReplace==null?null:txtReplace.getText());
		}
		return null;
	}
	
}

