/* 	PropertyDialog.java
 *	Copyright (c) 2001-2007, Brains2B.org
 *	
 *	Created by: Dennis Groenendijk
 *	Created on: March 27, 2001
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that  * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/* Changelog
 * When			Who 	What
 * --------------------------------------------------------------
 * 2002/01/12	Dennis	Revamped and automated setting properties
 * 						to components; Changed name to PropertyDialog.java
 * 2003/07/09	dennis	Rewrote the change procedure.
 * 						Added language support and definition of preference labels
 * 2005-06-02	dennis	#742 Prevent collapse of lists by setting prototype, 
 * 							 getComponent should check for preferred type first
 * 						Use EXIT_ON_CLOSE for main()
 * 2006-05-15	dennis@brains2b.nl ignore pattern now also works on tabbed pages.
 * 2006-09-02	dennis@brains2b.nl improved handling of the LIST option
 * 2006-11-28   dennis@brains2b.nl allow multiple ignore keys, desynchronized maps and list
 * 2007-05-29   dennis@brains2b.nl remove tabs if key is in ignore pattern						
 */

package org.brains2b.swingext.preference;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.MissingResourceException;
import java.util.Properties;
import java.util.ResourceBundle;
import java.util.Set;
import java.util.StringTokenizer;
import java.util.TreeSet;
import java.util.Vector;

import javax.swing.DefaultListModel;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JColorChooser;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JFileChooser;
import javax.swing.JInternalFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import javax.swing.SwingConstants;
import javax.swing.SwingUtilities;
import javax.swing.border.BevelBorder;
import javax.swing.border.EmptyBorder;
import javax.swing.event.CaretListener;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import org.brains2b.collection.EntryImpl;
import org.brains2b.swingext.dialog.DialogHelper;
import org.brains2b.swingext.fontchooser.FontChooserPane;
import org.brains2b.swingext.form.Form;
import org.brains2b.swingext.form.FormAction;
import org.brains2b.swingext.util.FontHelper;
import org.brains2b.util.GuiHelper;
import org.brains2b.util.StringHelper;

/** Default property dialog for application properties
 * Sets fields on basis of properties, overwrite initValues() to
 * set up your own content and properties.
 * <BR> 
 * Only use the getComponent() function[s] to set the properties,
 * otherwise properties will not not be set
 * @author  dennis
 * @version 0.52
 * @since 2006-11-28
 */
public class PreferencePane extends JPanel implements Form {

	private class ColorIcon extends ImageIcon {
		public final static int WIDTH=64;
		public final static int HEIGHT=16;
		public Color color;
		
		
		public ColorIcon(Color c) {
			color=c;
		}
		
		/* (non-Javadoc)
		 * @see javax.swing.ImageIcon#getIconHeight()
		 */
		public int getIconHeight() {
				return HEIGHT;
		}

		/* (non-Javadoc)
		 * @see javax.swing.ImageIcon#getIconWidth()
		 */
		public int getIconWidth() {
			return WIDTH;
		}

		
		/* (non-Javadoc)
		 * @see javax.swing.ImageIcon#paintIcon(java.awt.Component, java.awt.Graphics, int, int)
		 */
		public synchronized void paintIcon(
			Component c,
			Graphics g,
			int x,
			int y) {
			g.setColor(color);
			g.fillRect(x,y,WIDTH,HEIGHT);
			
		}

	}
	
	/* Values to determine what kind of component best represents
	   a given property */
	private final static int CHECKBOX = 1;
	private final static int COMBOBOX = 2;
	public final static int COLOR = 3;
	public final static int FONT = 4;
	private final static int FIELD = 5;
	public final static int FILE=6;
	public final static int LIST=7;
	
	private class ActionEventHandler implements ActionListener {
		/*
		 * @see ActionListener#actionPerformed(ActionEvent)
		 */
		public void actionPerformed(ActionEvent e) {
			if (e.getActionCommand().equals("color")) {
				openColorChooser((JComponent)e.getSource());
			} else if (e.getActionCommand().equals("font")) {
				openFontChooser((JComponent) e.getSource());
			} else if (e.getActionCommand().equals("file")){
				openFileChooser((JComponent) e.getSource());
			} else if(e.getActionCommand().equals("listadd")) {
				JButton b=(JButton) e.getSource();
				JTextField fld=(JTextField) b.getClientProperty("field");
				JList list=(JList) b.getClientProperty("list");
				((DefaultListModel) list.getModel()).addElement(fld.getText());
				fld.setText("");
				m_changed.add(list.getName());
			}else if(e.getActionCommand().equals("listremove")) {
				JButton b=(JButton) e.getSource();
				JList list=(JList) b.getClientProperty("list");
				((DefaultListModel) list.getModel()).remove(list.getSelectedIndex());
				m_changed.add(list.getName());
			}
		}

	}
	private class ValueChanged implements CaretListener, ChangeListener, ActionListener {

		public void caretUpdate(javax.swing.event.CaretEvent e) {
			try {
				JComponent comp = ((JComponent) e.getSource());
				if (!m_prop.getProperty(comp.getName()).equals(getValue(comp))) {
					m_changed.add(comp.getName());
				} else {
					m_changed.remove(comp.getName());
				}
			} catch (Exception pe) {
				//System.out.println("Error parsing integer");
				pe.printStackTrace();
			}
		}

			/*
		 * @see ChangeListener#stateChanged(ChangeEvent)
		 */
		public void stateChanged(ChangeEvent e) {
			try {
				JComponent comp = ((JComponent) e.getSource());
				if (!m_prop.getProperty(comp.getName()).equals(getValue(comp))) {
					m_changed.add(comp.getName());
				} else {
					m_changed.remove(comp.getName());
				}
			} catch (Exception pe) {
				//System.out.println("Error parsing integer");
				pe.printStackTrace();
			}
		}

		/**
		 * @see java.awt.event.ActionListener#actionPerformed(ActionEvent)
		 */
		public void actionPerformed(ActionEvent e) {
			try {
				JComponent comp = ((JComponent) e.getSource());
				if (!m_prop.getProperty(comp.getName()).equals(getValue(comp))) {
					m_changed.add(comp.getName());
				} else {
					m_changed.remove(comp.getName());
				}
			} catch (Exception pe) {
				//System.out.println("Error parsing integer");
				pe.printStackTrace();
			}
		}

}

	private JComponent mMain;
	private ResourceBundle m_resource;
	private TreeSet m_changed;
	private ValueChanged mValueChanged;
	private java.util.Properties m_prop;
	private ActionEventHandler mAeh;
	private HashMap propertyType;
	private boolean m_autoTab;
	private HashMap m_tabs;
	private ArrayList m_ignore;
	private ImageIcon fileIcon;
	private ImageIcon plusIcon;
	private ImageIcon minusIcon;
	
	private HashMap m_options;
	
	public PreferencePane(Properties prop) {
		super();
        propertyType=new HashMap();
        m_prop = prop;	
	}
	
	protected void initValues() {
		Set s = m_prop.keySet();
        TreeSet ts = new TreeSet(s);
        if (ts.isEmpty()) { //Build one from the information we have
            ts.addAll(m_options.keySet());
            ts.addAll(propertyType.keySet());
        }
		
		
		if (isAutoTab()) {
			makeTabs(ts);
		}
	
		JPanel pnl = new JPanel();
		pnl.setLayout(new GridBagLayout());
		pnl.setBorder(new EmptyBorder(5, 5, 5, 5));
		GridBagConstraints gbc=new GridBagConstraints();
		gbc.gridy=0;
		gbc.anchor=GridBagConstraints.WEST;
		gbc.insets=new Insets(1,3,1,3);
		boolean componentSet=false;
		Iterator it = ts.iterator();
		while (it.hasNext()) {
			String key = (String) it.next();
			if (!isIgnorePattern(key)) {
			    int idx=key.indexOf('.');
			    if (m_tabs==null || (m_tabs!=null && idx!=-1 && !m_tabs.containsKey(key.substring(0,idx)))) {
			        String label=getResourceValue(key,key);
			        setComponent(pnl,gbc,key, m_prop.getProperty(key), label);
			        componentSet=true;
			    }
			}
		}
		if (m_tabs==null || m_tabs.isEmpty()) {
			mMain=pnl;
		} else if (componentSet){
			addTabPane(pnl, getResourceValue("Preference.generalTitle","General"), null);
		}
		if (m_tabs!=null && !m_tabs.isEmpty()) {
			Iterator ittab=m_tabs.keySet().iterator();
			while(ittab.hasNext()) {
				String tabKey=(String) ittab.next();
				pnl = new JPanel();
				pnl.setLayout(new GridBagLayout());
				pnl.setBorder(new EmptyBorder(5, 5, 5, 5));
				gbc=new GridBagConstraints();
				gbc.gridy=0;
				gbc.anchor=GridBagConstraints.WEST;
				gbc.insets=new Insets(1,3,1,3);
		
				it = ts.iterator();
				while (it.hasNext()) {
					String key = (String) it.next();
					if (!isIgnorePattern(key)) {
						int idx=key.indexOf('.');
						if (idx!=-1 && tabKey.equals(key.substring(0,idx))) {
							String label=getResourceValue(key,key);
							setComponent(pnl,gbc,key, m_prop.getProperty(key), label);
						}
					}
				}
				addTabPane(pnl, (String) m_tabs.get(tabKey), null);
			}
		}
		
	}
	
	private JComponent findComponent(String name) {
		return findComponent(mMain,name);
	}

	private JComponent findComponent(Container root, String name) {
		JComponent cmp=null;
		Component[] cmps = root.getComponents();
    	for (int i = 0; i < cmps.length; i++) {
    		if (cmps[i] instanceof JPanel&& !name.equals(cmps[i].getName())) {
    			cmp=findComponent((JPanel) cmps[i],name);
    			if (cmp!=null) break;
    		} else if (cmps[i] instanceof JScrollPane && name.equals(((JScrollPane) cmps[i]).getViewport().getView().getName())) {
    			cmp=(JComponent)((JScrollPane) cmps[i]).getViewport().getView();
    		} else if (name.equals(cmps[i].getName())) {
    			cmp=(JComponent) cmps[i];
    			break;
    		}
    	}
    	return cmp;
	}

	private String getValue(JComponent cmp) {
		String value=null;
		if (cmp instanceof JTextField) {
			value = ((JTextField) cmp).getText();
		} else if (cmp instanceof JCheckBox) {
			value = String.valueOf(((JCheckBox)cmp).isSelected());
		} else if (cmp instanceof JButton) {
			JButton b= (JButton) cmp;
			if (b.getActionCommand().equals("font")) {
				value=FontHelper.encode(b.getFont());
			} else {
				value=String.valueOf(((ColorIcon)b.getIcon()).color.getRGB());
			}
		} else if (cmp instanceof JList) {
			value=getString(((DefaultListModel) ((JList)cmp).getModel()).toArray());
		} else if (cmp instanceof JComboBox) {
			value= ((EntryImpl) ((JComboBox) cmp).getSelectedItem()).getKey();
		}
		return value;
	}

	private String getValue(String key) {
		return getValue(findComponent(key));
	}

	private void setProperties() {

		if (!m_changed.isEmpty()) {
			Iterator it=m_changed.iterator();
			while (it.hasNext()) {
				String key= (String) it.next();
				m_prop.put(key,getValue(key));
			}
		}

	}

	/** This method is called from within the constructor to
	 * initialize the form.
	 * 
	 */
	private void init() {
        setLayout(new BorderLayout());
		mValueChanged = new ValueChanged();
		mMain = new JTabbedPane(SwingConstants.TOP);
		mMain.setBorder(new EmptyBorder(5, 5, 5, 5));
		mAeh=new ActionEventHandler();
		fileIcon=new ImageIcon(ClassLoader.getSystemResource("org/brains2b/images/Open16.gif"));
		plusIcon=new ImageIcon(ClassLoader.getSystemResource("org/brains2b/images/Plus16.gif"));
		minusIcon=new ImageIcon(ClassLoader.getSystemResource("org/brains2b/images/Minus16.gif"));
		
		//txt = new JComponent[psz];
		//bool = new boolean[psz];
		m_changed=new TreeSet();
		//System.out.println(psz);
		initValues();
		add(mMain, BorderLayout.CENTER);
		if (m_resource!=null) {
			add(new FormAction(this, 7,m_resource), BorderLayout.SOUTH);
		} else {
			add(new FormAction(this,FormAction.BTN_OK_CANCEL),BorderLayout.SOUTH);
		}
	}

	/*
	 * @see Form#performApply()
	 */
	public void performApply() {
		setProperties();
	}

	/*
	 * @see Form#performOk()
	 */
	public void performOk() {
		setProperties();
		close();
	}

	/*
	 * @see Form#performUndo()
	 */
	public void performUndo() {
		mMain.removeAll();
		initValues();
		validate();
	}

	/*
	 * @see Form#performCancel()
	 */
	public void performCancel() {
		close();
	}

	private void close() {
		if (getRootPane().getParent() instanceof JInternalFrame) {
            ((JInternalFrame) getRootPane().getParent()).doDefaultCloseAction();      
        } else {
            SwingUtilities.getWindowAncestor(this).dispose();
        }
	}

	protected void addTabPane(JPanel pnl, String title, Icon icon) {
		((JTabbedPane) mMain).addTab(title, icon, pnl);
	}

	protected void setComponent(JPanel p, GridBagConstraints gbc, String key, String value, String label) {
		//JComponent comp = null;
		
		switch (getComponentType(key, value)) {
			case CHECKBOX :
				setCheckBox(p,gbc,key, value, label);
				break;
			case COLOR :
				setColorPanel(p,gbc,key,value,label);
				break;
			case FONT :
				setFontPanel(p,gbc,key,value,label);
				break;
			case COMBOBOX :
				setComboBox(p,gbc,key,value,label);
				break;
			case FILE:
				setFilePanel(p,gbc,key,value,label);
				break;
			case LIST:	
				setListPanel(p,gbc,key,value,label);
				break;
			case FIELD :
			default :
				setTextField(p,gbc,key, value, label);
				break;
		}
		
		//return comp;
	}

	protected void setTextField(JPanel p, GridBagConstraints gbc,String key, String value, String label) {
		JLabel lbl = new JLabel(label);
		lbl.setForeground(Color.black);
		JTextField fld = new JTextField();
		fld.setName(key);
		if (value != null && value.length() > 0) {
			fld.setColumns(value.length() + 3);
		} else {
			fld.setColumns(10);
		}
		fld.addCaretListener(mValueChanged);
		fld.setName(key);
		fld.setText(value);

		lbl.setLabelFor(fld);
		p.add(lbl,gbc);
		gbc.gridx=1;
		gbc.gridwidth=2;
		p.add(fld,gbc);
		gbc.gridwidth=1;
		gbc.gridx=0;
		gbc.gridy++;
	}

	protected void setColorPanel(JPanel p, GridBagConstraints gbc,String key, String value, String label) {
		JLabel lbl=new JLabel(label);
		JButton btn=new JButton();
        Color c= Color.white;
        if (value!=null) c= Color.decode(value);
        
		btn.setIcon(new ColorIcon(c));
		btn.setName(key);
		btn.addActionListener(mAeh);
		btn.setActionCommand("color");
		
		p.add(lbl,gbc);
		gbc.gridx=1;
		gbc.gridwidth=2;
		p.add(btn,gbc);
		gbc.gridy++;
		gbc.gridx=0;
		gbc.gridwidth=1;
	}

	protected void setFilePanel(JPanel p, GridBagConstraints gbc,String key, String value, String label) {
		JLabel lbl=new JLabel(label);
		JTextField file=new JTextField();
		file.setColumns(32);
		file.setBorder(new BevelBorder(BevelBorder.LOWERED));
		file.setText(value);
		file.addCaretListener(new ValueChanged());
		file.setName(key);
		JButton btn=new JButton(fileIcon);
		btn.addActionListener(mAeh);
		btn.setName(key);
		btn.setActionCommand("file");
		p.add(lbl,gbc);
		gbc.gridx=1;
		p.add(file,gbc);
		gbc.gridx=2;
		p.add(btn,gbc);
		gbc.gridy++;
		gbc.gridx=0;
		gbc.gridwidth=1;
	}
	
	protected void setListPanel(JPanel p, GridBagConstraints gbc,String key, String value, String label) {
			JPanel pnl=new JPanel();
			pnl.setLayout(new FlowLayout());
			JLabel lbl=new JLabel(label);
			JTextField entry=new JTextField();
			entry.setBorder(new BevelBorder(BevelBorder.LOWERED));
			JList list=new JList();
			DefaultListModel model= new DefaultListModel();
			list.setModel(model);
			list.setMinimumSize(new Dimension(80,40));
			list.setName(key);
			list.setPrototypeCellValue(StringHelper.leftPad("M","M",16));
			list.setVisibleRowCount(5);
			list.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
			if (value!=null) {
				String items[] = getList(value);
				for (int i=0;i<items.length;i++) {
					model.addElement(items[i]);
				}
			}
			JScrollPane scroll=new JScrollPane(list);
			JButton add=new JButton(plusIcon);
			add.putClientProperty("field",entry);
			add.putClientProperty("list",list);
			add.addActionListener(mAeh);
			//add.setName(key);
			add.setActionCommand("listadd");
			JButton remove=new JButton(minusIcon);
			remove.putClientProperty("list",list);
			remove.addActionListener(mAeh);
			//remove.setName(key);
			remove.setActionCommand("listremove");
			
			p.add(lbl,gbc);
			gbc.gridx=1;
			gbc.fill=GridBagConstraints.HORIZONTAL;
			p.add(entry,gbc);
			gbc.fill=GridBagConstraints.NONE;
			gbc.gridx=2;
			p.add(add,gbc);
			gbc.gridy++;
			gbc.gridx=1;
			gbc.fill=GridBagConstraints.HORIZONTAL;
			p.add(scroll,gbc);
			gbc.fill=GridBagConstraints.NONE;
			gbc.gridx=2;
			gbc.anchor=GridBagConstraints.NORTHWEST;
			p.add(remove,gbc);
			gbc.anchor=GridBagConstraints.WEST;
			gbc.gridy++;	
			gbc.gridx=0;
			gbc.gridwidth=1;
		}

	protected void setFontPanel(JPanel p, GridBagConstraints gbc,String key, String value, String label) {
		JLabel lbl=new JLabel(label);
		/*JLabel font=new JLabel();*/
		/*font.setText(value);
		font.setName(key);*/
		//font.putClientProperty("index", new Integer(mIndex));
		//txt[mIndex++]=font;
		JButton btn=new JButton();
		btn.setFont(Font.decode(value));
		btn.setText(value);
		btn.setName(key);
		btn.addActionListener(mAeh);
		//btn.putClientProperty("target",font);
		btn.setActionCommand("font");
		p.add(lbl,gbc);
		gbc.gridx=1;
		gbc.fill=GridBagConstraints.BOTH;
		/*p.add(font,gbc);
		gbc.gridx=2;
		;*/
		p.add(btn,gbc);
		gbc.fill=GridBagConstraints.NONE;
		gbc.gridy++;
		gbc.gridx=0;
		gbc.gridwidth=1;
	}
	
	protected void setCheckBox(JPanel p, GridBagConstraints gbc, String key, String value, String label) {
		JCheckBox chk = new JCheckBox();
		chk.setName(key);
		chk.setText(label);
		chk.setSelected(new Boolean(value).booleanValue());
		chk.addChangeListener(mValueChanged);
		chk.setName(key);
		gbc.gridx=1;
		gbc.gridwidth=2;
		p.add(chk,gbc);
		gbc.gridwidth=1;
		gbc.gridx=0;
		gbc.gridy++;
	}

	protected void setComboBox(JPanel p, GridBagConstraints gbc,String key, String value, String label) {
		JComboBox cmb=new JComboBox((Vector) m_options.get(key));
		JLabel lbl = new JLabel(label);
		lbl.setForeground(Color.black);
		cmb.addActionListener(mValueChanged);
		cmb.setName(key);
		for (int i=0;i<cmb.getItemCount();i++) {
			if (cmb.getItemAt(i).equals(value)) {
				cmb.setSelectedIndex(i); break;
			}
		}
		
		lbl.setLabelFor(cmb);
		p.add(lbl,gbc);
		gbc.gridx=1;
		p.add(cmb,gbc);
		gbc.gridx=0;
		gbc.gridy++;
	}

	private int getComponentType(String key, String value) {
		int result = 0;
		if (propertyType.containsKey(key)) {
			result = ((Integer) propertyType.get(key)).intValue();
		} else if (value!=null && (value.equalsIgnoreCase("true") || value.equalsIgnoreCase("false"))) {
			result = CHECKBOX;
		} else if (keyHasOptions(key)) {
			result = COMBOBOX;
		} else if (key.toLowerCase().indexOf("font") != -1 && GuiHelper.isFont(value)) {
			result = FONT;
		} else if (key.toLowerCase().indexOf("color") != -1 && GuiHelper.isColor(value)) {
			result = COLOR;
		} else if (value.indexOf(System.getProperty("file.separator"))!=-1) {
			result=FILE;
		} 
		return result;
	}

	private boolean keyHasOptions(String key) {
		if (m_options!=null && m_options.get(key)!=null) {
			return true;
		}
		return false;
			
	}
    protected void firePropertyChange(String propertyName, Object oldValue, Object newValue) {
        super.firePropertyChange(propertyName, oldValue, newValue);
        if ("ancestor".equals(propertyName) && oldValue==null) {
            init();
        }
        
    }
    
	/**
	 * Gets the changed.
	 * @return Returns a boolean
	 */
	public boolean isChanged() {
		return !m_changed.isEmpty();
	}

	private void openColorChooser(JComponent c) {
		JButton target =(JButton) getTarget(c);
		Color color=JColorChooser.showDialog(this,"Pick a color",target.getBackground());
		if ( color!=null) {
			//target.setBackground(color);
			target.setIcon(new ColorIcon(color));
			m_changed.add(target.getName());
		}
	}
	
	private void openFontChooser(JComponent c) {
		JButton target= (JButton) getTarget(c);
		Font f=FontChooserPane.showDialog(this,target.getFont(),"Pick a font");
		if (f!=null) {
			target.setFont(f);
			target.setText(FontHelper.encode(f));
			m_changed.add(target.getName());
		}
	}
	
	private void openFileChooser(JComponent c) {
		JTextField target=(JTextField) _getTarget(c);
		JFileChooser jfc=new JFileChooser();
		if (target.getText()!=null) {
			File f=new File(target.getText());
			if (f.exists()) jfc.setCurrentDirectory(f);
		}
		jfc.setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES);
		jfc.showOpenDialog(target);
		File f=jfc.getSelectedFile();
		if (f!=null) {
			target.setText(f.getAbsolutePath());
			m_changed.add(target.getName());
		}
	}
	
	private Component getTarget(JComponent c) {
		if (c.getClientProperty("target")==null) {
			return c;
		} 
		return (JComponent) c.getClientProperty("target");
	}
	
	private Component _getTarget(JComponent c) {
		Container parent=c.getParent();
		
		for (int i=0;i<parent.getComponentCount();i++) {
			Component child=parent.getComponent(i);
			if (child!=null && !c.equals(child) && c.getName().equals(child.getName())) {
				return child;
			}
		}
		return null;
	}
	
	public void setIgnorePattern(String pattern) {
	    if (m_ignore==null) {
            m_ignore = new ArrayList();
        } else {
            m_ignore.clear();
        }
        m_ignore.add(pattern);
	}
    
    public void addIgnorePattern(String pattern) {
        if (m_ignore==null) m_ignore = new ArrayList();
            m_ignore.add(pattern);
    }
    
    private boolean isIgnorePattern(String key) {
        if (m_ignore==null) return false;
        for (Iterator it=m_ignore.iterator();it.hasNext();) {
            if (key.indexOf((String)it.next())!=-1) {
                return true;
            }
        }
        return false;
    }
	
	public void setPropertyType(String key, int type) {
		propertyType.put(key,new Integer(type));
	}
	
	public void setOptionsFor(String key, Hashtable t) {
		if (m_options==null) {
			m_options=new HashMap();
		}
		Vector v=new Vector();
		Iterator it = t.keySet().iterator();
		while (it.hasNext()) {
			String itm= (String) it.next();
			v.add(new EntryImpl(itm,(String) t.get(itm)));
		}
		m_options.put(key,v);
	}
	
	public void setOptionsFor(String key, String[] options) {
		if (m_options==null) {
			m_options=new HashMap();
		}
		Vector v=new Vector();
		
		for (int i=0;i<options.length;i++) {
			v.add(new EntryImpl(options[i],options[i]));
		}
		m_options.put(key,v);
	}
	
	public void setOptionsFor(String key, String[][] options) {
		if (m_options==null) {
			m_options=new HashMap();
		}
		Vector v=new Vector();
		
		for (int i=0;i<options.length;i++) {
			v.add(new EntryImpl(options[i][0],options[i][1]));
		}
		m_options.put(key,v);
	}
	
    /**
     * Registers a resourceBundle for this component. 
     * <p>This resource bundle will be used to set the labels for the individual
     * preferences in this screen. The key of the resource bundle should be the same
     * as the key for the preference.
     * <p>The title of the General tab can be changed by supplying a key named
     * <code>Preference.generalTitle</code>
     * <BR>The <i>Choose...</i> button can be set by <code>Preference.btnChoose</code>
     * <p>This resourceBundle is passed to the FormAction component used here to use with
     * the labels on the buttons. Look at FormAction for key names and content
     * @param resource, ResourceBundle
     * @since Framework 0.31
     * @see org.brains2b.model.gui.FormAction
     */
	public void registerResourceBundle(ResourceBundle resource) {
		m_resource=resource;
	}
	
    /**
     * returns the String from the resource bundle for a given key.
     * <p>Returns the defaultValue if no resource bundle was registered or
     * if the key cannot be found in the resource bundle.
     * @param key String, the key to find in the resource bundle
     * @param defaultValue String, the default value if no bundle or key was defined
     * @return String, the Value found in the resourcebundle or the defaultValue
     * @since Framework 0.31
     */
	protected String getResourceValue(String key, String defaultValue) {
		String label=defaultValue;
		if (m_resource!=null) {
			try {
				label=m_resource.getString(key);
			} catch (MissingResourceException mrex) {
			}
		}
		return label;
	}
	
	public static final String[] getList(String s) {
		if (s==null) return null;
		StringTokenizer t=new StringTokenizer(s,";");
		ArrayList v=new ArrayList();
		while (t.hasMoreTokens()) {
			v.add(t.nextToken().trim());
		}
		String[] list=new String[v.size()];
		v.toArray(list);
		return list;
	}
	
	private String getString(Object[] list) {
		StringBuffer sb=new StringBuffer();
		for (int i=0;i<list.length;i++) {
			if (i>0) sb.append("; ");
			sb.append(list[i]);
		}
		return sb.toString();
	}
	
	/**
	 * @return
	 */
	public boolean isAutoTab() {
		return m_autoTab;
	}

	/**
	 * @param b
	 */
	public void setAutoTab(boolean b) {
		m_autoTab = b;
	}

	public void addTab(String forPrefix, String label) {
		if (m_tabs==null) {
			m_tabs=new HashMap();
		}
		m_tabs.put(forPrefix,label);
	}
	
	protected void makeTabs(TreeSet s) {
		if (m_tabs==null) m_tabs=new HashMap();
		Iterator it=s.iterator();
		while (it.hasNext()) {
			String key=(String) it.next();
			int idx=key.indexOf(".");
            String tabKey =key.substring(0,idx==-1?key.length():idx);
			if (idx!=-1 && m_tabs!=null && !m_tabs.containsKey(tabKey) && !m_ignore.contains(tabKey)) {
				m_tabs.put(tabKey,tabKey);
			}
		}
	}

    public static Properties showDialog(Component parent, Properties p) {
    	PreferencePane pane = new PreferencePane(p);
        DialogHelper.showDialog(parent,pane,"Connections");
        return p;
    }

    public static Properties showInternalDialog(Component parent, Properties p, String title, Icon icon) {
        PreferencePane pane = new PreferencePane(p);
        DialogHelper.showInternalDialog(parent,pane,title,icon);
        return p;
    }
}